@@ -10,12 +10,13 @@
 
 import contextlib
 from abc import ABC, abstractmethod
-from dataclasses import dataclass
+from dataclasses import asdict, dataclass
 from typing import Any, Callable, Optional, Tuple, TypeVar, Union
 
 import torch
 from pyre_extensions import none_throws
 from torch.cuda.amp import GradScaler
+from torch.distributed import ProcessGroup
 from torch.distributed.fsdp import FullyShardedDataParallel as FSDP
 from torch.nn.parallel import DistributedDataParallel as DDP
 from torch.optim.swa_utils import AveragedModel, SWALR
@@ -23,19 +24,41 @@ from torchtnt.framework.state import EntryPoint, State
 from torchtnt.framework.unit import EvalUnit, PredictUnit, TrainUnit
 from torchtnt.utils import (
     copy_data_to_device,
-    get_device_from_env,
+    init_from_env,
     is_torch_version_geq_1_12,
-    maybe_enable_tf32,
     TLRScheduler,
     transfer_batch_norm_stats,
     transfer_weights,
 )
+from torchtnt.utils.rank_zero_log import rank_zero_warn
 from torchtnt.utils.version import is_torch_version_ge_1_13_1
 from typing_extensions import Literal
 
 TSWA_avg_fn = Callable[[torch.Tensor, torch.Tensor, int], torch.Tensor]
 
 
+@dataclass
+class Strategy:
+    """Dataclass representing the parallelization strategy for the AutoParallelUnit"""
+
+    pass
+
+
+@dataclass
+class DDPStrategy(Strategy):
+    """Dataclass representing the `DistributedDataParallel <https://pytorch.org/docs/stable/generated/torch.nn.parallel.DistributedDataParallel.html>`_ strategy"""
+
+    output_device: Optional[Union[int, torch.device]] = None
+    dim: int = 0
+    broadcast_buffers: bool = True
+    process_group: Optional[ProcessGroup] = None
+    bucket_cap_mb: int = 25
+    find_unused_parameters: bool = False
+    check_reduction: bool = False
+    gradient_as_bucket_view: bool = False
+    static_graph: bool = False
+
+
 @dataclass
 class SWAParams:
     """
@@ -76,12 +99,15 @@ TData = TypeVar("TData")
 class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
     """
     The AutoUnit is a convenience for users who are training with stochastic gradient descent and would like to have model optimization
-    handled for them. The AutoUnit subclasses :class:`~torchtnt.framework.unit.TrainUnit`, :class:`~torchtnt.framework.unit.EvalUnit`, and :class:`~torchtnt.framework.unit.PredictUnit`, and implements the ``train_step``, ``eval_step``, and ``predict_step`` for the user.
+    and data parallel replication handled for them.
+    The AutoUnit subclasses :class:`~torchtnt.framework.unit.TrainUnit`, :class:`~torchtnt.framework.unit.EvalUnit`, and :class:`~torchtnt.framework.unit.PredictUnit`,
+    and implements the ``train_step``, ``eval_step``, and ``predict_step`` methods for the user.
 
     For the ``train_step`` it runs:
-    * forward pass as part of the loss computation
-    * backward pass
-    * optimizer step
+
+    - forward pass and loss computation
+    - backward pass
+    - optimizer step
 
     For the ``eval_step`` it only runs forward and loss computation.
 
@@ -91,12 +117,12 @@ class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
     Then use with the :py:func:`~torchtnt.framework.train`, :py:func:`~torchtnt.framework.evaluate`, :py:func:`~torchtnt.framework.predict`, or :py:func:`~torchtnt.framework.fit` entry point as normal.
 
     For more advanced customization, directly use the :class:`~torchtnt.framework.unit.TrainUnit`, :class:`~torchtnt.framework.unit.EvalUnit`, and :class:`~torchtnt.framework.unit.PredictUnit` interfaces.
+
     Args:
         module: module to be used during training.
-        optimizer: optimizer to be used during training.
-        lr_scheduler: lr_scheduler to be used during training.
-        step_lr_interval: whether to step lr_scheduler every step or every epoch. Defaults to every epoch.
         device: the device to be used.
+        strategy: the data parallelization strategy to be used
+        step_lr_interval: whether to step lr_scheduler every step or every epoch. Defaults to every epoch.
         log_frequency_steps: how often to log in terms of steps (parameter updates) during training.
         precision: the precision to use in training, as either a string or a torch.dtype.
         gradient_accumulation_steps: how many batches to accumulate gradients over.
@@ -104,24 +130,7 @@ class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
         clip_grad_norm: max norm of the gradients for clipping https://pytorch.org/docs/stable/generated/torch.nn.utils.clip_grad_norm_.html
         clip_grad_value: max value of the gradients for clipping https://pytorch.org/docs/stable/generated/torch.nn.utils.clip_grad_value_.html
         swa_params: params for stochastic weight averaging https://pytorch.org/docs/stable/optim.html#stochastic-weight-averaging
-        torchdynamo_params: params for TorchDynamo
-
-    Attributes:
-        module: module to be used during training.
-        optimizer: optimizer to be used during training.
-        lr_scheduler: lr_scheduler to be used during training.
-        step_lr_interval: whether to step lr_scheduler every step or every epoch. Defaults to every epoch.
-        device: the device to be used.
-        log_frequency_steps: how often to log in terms of steps (parameter updates) during training.
-        precision: the precision to use in training, as a torch.dtype.
-        grad_scaler: a torch.cuda.amp.GradScaler, if using fp16 precision.
-        gradient_accumulation_steps: how many batches to accumulate gradients over.
-        num_optimizer_steps_completed: number of optimizer steps (weight updates) completed.
-        detect_anomaly: whether to enable anomaly detection for the autograd engine https://pytorch.org/docs/stable/autograd.html#anomaly-detection
-        clip_grad_norm: max norm of the gradients for clipping https://pytorch.org/docs/stable/generated/torch.nn.utils.clip_grad_norm_.html
-        clip_grad_value: max value of the gradients for clipping https://pytorch.org/docs/stable/generated/torch.nn.utils.clip_grad_value_.html
-        swa_params: params for stochastic weight averaging https://pytorch.org/docs/stable/optim.html#stochastic-weight-averaging
-        torchdynamo_params: params for TorchDynamo
+        torchdynamo_params: params for TorchDynamo https://pytorch.org/docs/master/dynamo/
 
             Note:
                 TorchDynamo support is only available in PyTorch 2.0 or higher.
@@ -131,10 +140,9 @@ class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
         self,
         *,
         module: torch.nn.Module,
-        optimizer: torch.optim.Optimizer,
-        lr_scheduler: Optional[TLRScheduler] = None,
-        step_lr_interval: Literal["step", "epoch"] = "epoch",
         device: Optional[torch.device] = None,
+        strategy: Optional[Strategy] = None,
+        step_lr_interval: Literal["step", "epoch"] = "epoch",
         log_frequency_steps: int = 1000,
         precision: Optional[Union[str, torch.dtype]] = None,
         gradient_accumulation_steps: int = 1,
@@ -145,12 +153,30 @@ class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
         torchdynamo_params: Optional[TorchDynamoParams] = None,
     ) -> None:
         super().__init__()
-        self.module = module
-        self.optimizer = optimizer
-        self.lr_scheduler = lr_scheduler
+        self.device: torch.device = device or init_from_env()
+
+        if strategy:
+            if isinstance(strategy, DDPStrategy):
+                # move module to device
+                module = module.to(self.device)
+                # wrap module in DDP
+                device_ids = None
+                if self.device.type == "cuda":
+                    device_ids = [self.device.index]
+                module = DDP(module, device_ids=device_ids, **asdict(strategy))
+                if torchdynamo_params:
+                    # TODO: Add support for dynamo and DDP
+                    rank_zero_warn(
+                        "Torchdynamo params has been set with DDP - Note that performance will likely be slower and we recommend using only one."
+                    )
+
+        else:
+            # move module to device
+            module = module.to(self.device)
+
+        self.module: torch.nn.Module = module
+
         self.step_lr_interval = step_lr_interval
-        self.device: torch.device = device or get_device_from_env()
-        maybe_enable_tf32()
         if not log_frequency_steps > 0:
             raise ValueError(
                 f"log_frequency_steps must be > 0. Got {log_frequency_steps}"
@@ -192,25 +218,7 @@ class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
             enabled=self.precision is not None,
         )
 
-        self.swa_model: Optional[AveragedModel] = None
-        self.swa_scheduler: Optional[SWALR] = None
         self.swa_params: Optional[SWAParams] = swa_params
-        if swa_params:
-            if not swa_params.avg_fn:
-                # pyre-ignore: Unexpected keyword [28]
-                self.swa_model = AveragedModel(self.module, use_buffers=True)
-            else:
-                # pyre-ignore: Unexpected keyword [28]
-                self.swa_model = AveragedModel(
-                    self.module, avg_fn=swa_params.avg_fn, use_buffers=True
-                )
-
-            self.swa_scheduler = SWALR(
-                optimizer=self.optimizer,
-                swa_lr=swa_params.lr,
-                anneal_epochs=swa_params.anneal_epochs,
-                anneal_strategy=swa_params.anneal_strategy,
-            )
 
         if torchdynamo_params:
             if not is_torch_version_ge_1_13_1():
@@ -228,6 +236,22 @@ class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
 
         # TODO: Make AutoTrainUnit work when data type is Iterator
 
+    @abstractmethod
+    def configure_optimizers_and_lr_scheduler(
+        self, module: torch.nn.Module
+    ) -> Tuple[torch.optim.Optimizer, TLRScheduler]:
+        """
+        The user should implement this method with their optimizer and learning rate scheduler construction code. This will be called upon initialization of
+        the AutoUnit.
+
+        Args:
+            module: the module with which to construct optimizer and lr_scheduler
+
+        Returns:
+            A tuple containing optimizer and optionally the learning rate scheduler
+        """
+        ...
+
     @abstractmethod
     def compute_loss(self, state: State, data: TData) -> Tuple[torch.Tensor, Any]:
         """
@@ -260,8 +284,9 @@ class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
     ) -> None:
         """
         The user should implement this method with their code to log metrics. This will be called:
-        * every ``train_step`` based on ``log_frequency_steps`` and how many parameter updates have been run on the model
-        * in ``on_train_epoch_end`` and ``on_eval_epoch_end``
+
+        - every ``train_step`` based on ``log_frequency_steps`` and how many parameter updates have been run on the model
+        - in ``on_train_epoch_end`` and ``on_eval_epoch_end``
 
         Args:
             state: a State object which is passed from ``train_step``/``on_train_epoch_end``/``on_eval_epoch_end``
@@ -284,6 +309,39 @@ class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
         """
         return copy_data_to_device(data, self.device)
 
+    def on_train_start(self, state: State) -> None:
+        """
+        Note that if implementing `on_train_start()`, you must call `super().on_train_start()`.
+        """
+        optimizer, lr_scheduler = self.configure_optimizers_and_lr_scheduler(
+            self.module
+        )
+        self.optimizer: torch.optim.optimizer.Optimizer = optimizer
+        self.lr_scheduler: TLRScheduler = lr_scheduler
+
+        self.swa_model: Optional[AveragedModel] = None
+        self.swa_scheduler: Optional[SWALR] = None
+
+        if self.swa_params:
+            if not self.swa_params.avg_fn:
+                # pyre-ignore: Unexpected keyword [28]
+                self.swa_model = AveragedModel(self.module, use_buffers=True)
+            else:
+                # pyre-ignore: Unexpected keyword [28]
+                self.swa_model = AveragedModel(
+                    self.module, avg_fn=self.swa_params.avg_fn, use_buffers=True
+                )
+
+            self.swa_scheduler = SWALR(
+                optimizer=self.optimizer,
+                # pyre-ignore: Undefined attribute [16]
+                swa_lr=self.swa_params.lr,
+                # pyre-ignore: Undefined attribute [16]
+                anneal_epochs=self.swa_params.anneal_epochs,
+                # pyre-ignore: Undefined attribute [16]
+                anneal_strategy=self.swa_params.anneal_strategy,
+            )
+
     def train_step(self, state: State, data: TData) -> Tuple[torch.Tensor, Any]:
         data = self.move_data_to_device(state, data)
 
@@ -308,7 +366,7 @@ class AutoUnit(TrainUnit[TData], EvalUnit[TData], PredictUnit[Any], ABC):
 
     def _forward_and_backward(
         self, state: State, data: TData, should_update_weights: bool
-    ):
+    ) -> Tuple[torch.Tensor, Any]:
         # if using gradient accumulation and DDP or FSDP, when in a step where we will not update the weights,
         # run forward and backward in no_sync context
         # https://pytorch.org/docs/stable/_modules/torch/nn/parallel/distributed.html#DistributedDataParallel.no_sync
