@@ -10,6 +10,7 @@ from .lambda_callback import Lambda
 from .learning_rate_monitor import LearningRateMonitor
 from .module_summary import ModuleSummary
 from .pytorch_profiler import PyTorchProfiler
+from .system_resources_monitor import SystemResourcesMonitor
 from .tensorboard_parameter_monitor import TensorBoardParameterMonitor
 from .torchsnapshot_saver import TorchSnapshotSaver
 from .tqdm_progress_bar import TQDMProgressBar
@@ -22,6 +23,7 @@ __all__ = [
     "LearningRateMonitor",
     "ModuleSummary",
     "PyTorchProfiler",
+    "SystemResourcesMonitor",
     "TensorBoardParameterMonitor",
     "TorchSnapshotSaver",
     "TQDMProgressBar",
